# Revision 1.3
# 2024-01-24
from machine import UART, Pin
from time import sleep_ms, ticks_ms, ticks_diff

START       = 0x7E
version     = 0xFF
length      = 6
acknowledge = 0
END         = 0xEF

# Verzoegerungszeiten
confDelay = 1000
playDelay = 500
volDelay  = 500

def proof(x, minVal, maxVal):
    x=(x if x<=maxVal else maxVal)
    x=(x if x>=minVal else minVal)
    return x

def TimeOut(t):
    start=ticks_ms()
    def compare():
        return int(ticks_ms()-start) >= t
    return compare

class DFPlayer():
    def __init__(self, uart=None, txd=19, rxd=18, busyPinNbr=17, config=True, volume=50):
        self._volume = volume
        self.configtime=config
        if uart is None:
            self.uart = UART(1, 9600,tx=txd, rx=rxd) # UART on 
            self.uart.init(9600, bits=8, parity=None, stop=1, tx=txd, rx=rxd)
        else:
            self.uart = uart
        self.busyPin = Pin(busyPinNbr,Pin.IN, Pin.PULL_UP)
        if config:
            self.config()
        if volume is not None:
            self.volume(volume)
        self.playCmdDone=TimeOut(1)
        print("DF-Player Rev. 2.0\n",self.uart,"\nBusy-Pin:",busyPinNbr)

    def sendCommand(self, cmd, par1, par2, ack=0):
        self.isConfigDone()
        checkSum = -(version + length + cmd + ack + par1 + par2)
        chkHigh = (checkSum >> 8) &0xFF
        chkLow = checkSum & 0xFF

        block=[
            START, version, length, cmd, ack,
            par1, par2, chkHigh, chkLow, END
            ]
        cmdBlock = bytes([b & 0xFF for b in block])
        self.uart.write(cmdBlock)
#         for b in block:
#             print(hex(b)," ",end="")
#         print("\n",block,"\n",hex(checkSum),checkSum)

    def config(self):
        self.confCmdDone = TimeOut(confDelay)
        self.sendCommand(0x3F, 0x00, 0x00)
        
    def isConfigDone(self):
        while not self.confCmdDone():
            sleep_ms(10)

    def play(self, folder, track, ack=0):
        self.isConfigDone()
        self.playCmdDone = TimeOut(playDelay)
        self.sendCommand(0x0F, folder, track, ack)

    def finish(self, folder, track):
        self.play(folder, track)
        while self.isPlaying():
            sleep_ms(50)

    def isPlaying(self):
        self.isPlayCmdDone()
        return self.busyPin.value() == 0

    def isPlayCmdDone(self):
        while not self.playCmdDone():
            sleep_ms(10)

    def volume(self, volume=None, ack=0):
        self.isConfigDone()
        if volume is None:
            return self._volume
        else:
            self._volume = int(proof(volume, 0, 100))
            val = int(self._volume*30/100)
            self.sendCommand(0x06,0 ,val, ack)
            self.volCmdDone = TimeOut(volDelay)

    def isVolCmdDone(self):
        while not self.volCmdDone():
            sleep_ms(10)

    def standby(self):
        self.isConfigDone()
        self.sendCommand(0x0A, 0x00, 0x00)

    def wakeUp(self):
        self.isConfigDone()
        self.sendCommand(0x0B, 0x00, 0x00)

    def reset(self):
        self.isConfigDone()
        self.sendCommand(0x0C, 0x00, 0x00)

    def resume(self):
        self.isConfigDone()
        self.sendCommand(0x0D, 0x00, 0x00)

    def pause(self):
        self.isConfigDone()
        self.sendCommand(0x0E, 0x00, 0x00)
        
    def setGain(self, gain=100):
        self.isConfigDone()
        gain = proof(gain, 0, 100)
        val = int(gain/100*30)
        self.sendCommand(0x10,0 ,val)  

    def repeat(self, repeat=True):
        self.isConfigDone()
        val = 1 if repeat else 0
        self.sendCommand(0x11, 0, val)

    def stop(self) :
        self.sendCommand(0x16, 0, 0)
        
    def read(self):
        c=self.uart.read()
        if c is not None:
            frame = []
            for i in range(len(c)):
                frame.append(hex(c[i]))
            return frame
        return None

    def getVersion(self):
        self.uart.read()
        self.sendCommand(0x46,0,0,0)
        sleep_ms(500)
        rec=self.uart.read()
        if rec is not None:
            return rec.decode()
        return None
     
    def getNumberOfFiles(self):
        self.uart.read()
        self.sendCommand(0x48,0,0,0)
        sleep_ms(500)
        rec=self.uart.read()
        if rec is not None and rec[3] == 0x48:
            return rec[6]
        return None
        
    def getNumberOfFilesInFolder(self):
        self.uart.read()
        self.sendCommand(0x4E,0,0,0)
        sleep_ms(500)
        rec=self.uart.read()
        if rec is not None:
            return rec[6]
        return None

    def getNumberOfFolders(self):
        self.uart.read()
        self.sendCommand(0x4F,0,0,0)
        sleep_ms(500)
        rec=self.uart.read()
        if rec is not None:
            return rec[6]
        return None

    def getCurrentTrack(self):
        self.uart.read()
        self.sendCommand(0x4B,0,0,0)
        sleep_ms(500)
        rec=self.uart.read()
        if rec is not None and rec[3] == 0x4C:
            return rec[6]
        return None
    
    def getVolume(self):
        self.uart.read()
        self.sendCommand(0x43,0,0,0)
        sleep_ms(500)
        rec=self.uart.read()
        if rec is not None:
            return (rec[6] /30) *100
        return None
         
    def command(self,cmd,p1,p2,ack=1):
        self.uart.read()
        self.sendCommand(cmd,p1,p2,ack)
        sleep_ms(500)
        rec=self.uart.read()
        print("rec=",rec)
        if rec is not None:
            return rec[5],rec[6]
        return None
